import logging
from typing import List, Dict
from mlcube.common import mlcube_metadata
from kubernetes import client, config

volume_mount_prefix = "/mnt/mlcube/"


class KubernetesRun(object):
    logger: logging.Logger = None
    mlcube_job_manifest: client.V1Job = None
    namespace: str = "default"

    def __init__(self, mlcube: mlcube_metadata.MLCube, loglevel: str):
        """Kubernetes Runner.
        Args:
            mlcube (mlcube_metadata.MLCube): MLCube specification. Reuses platform config from Docker.
        """
        logging.basicConfig(format='%(asctime)s - %(message)s',
                            datefmt='%d-%b-%y-%H:%M:%S',
                            level=loglevel)
        self.logger = logging.getLogger(__name__)

        self.mlcube: mlcube_metadata.MLCube = mlcube
        logging.info("MLCube instantiated!")

    def binding_to_volumes(self, binding: Dict, args: List[str], volume_mounts: Dict, volumes: Dict):
        for key, value in binding.items():
            args.append("--" + key + "=" + volume_mount_prefix +
                                  value['k8s']['pvc'] + "/" +
                                  value['path'])
            volume_mounts[
                value['k8s']['pvc']] = client.V1VolumeMount(
                    name=value['k8s']['pvc'],
                    mount_path=volume_mount_prefix +
                    value['k8s']['pvc'],
                )
            volumes[value['k8s']['pvc']] = client.V1Volume(
                name=value['k8s']['pvc'],
                persistent_volume_claim=client.
                V1PersistentVolumeClaimVolumeSource(
                    claim_name=value['k8s']['pvc']))

    def create_job_manifest(self):
        image: str = self.mlcube.platform.container.image
        logging.info(f"Using image: {image}")

        container_args: List[str] = []
        container_volume_mounts: Dict = dict()
        container_volumes: Dict = dict()

        container_args.append(self.mlcube.invoke.task_name)
        self.binding_to_volumes(self.mlcube.invoke.input_binding, container_args, container_volume_mounts, container_volumes)
        self.binding_to_volumes(self.mlcube.invoke.output_binding, container_args, container_volume_mounts, container_volumes)

        logging.info("Using Container arguments: %s" % container_args)

        container = client.V1Container(name="mlcube-container",
                                       image=image,
                                       args=container_args,
                                       volume_mounts=list(
                                           container_volume_mounts.values()))
        pod_template = client.V1PodTemplateSpec(
            metadata=client.V1ObjectMeta(labels={
                "app": "mlcube",
                "app-name": self.mlcube.name,
            }),
            spec=client.V1PodSpec(restart_policy="Never",
                                  containers=[container],
                                  volumes=list(container_volumes.values())))
        job_spec = client.V1JobSpec(
            template=pod_template,
            backoff_limit=4,
        )

        self.mlcube_job_manifest = client.V1Job(
            api_version="batch/v1",
            kind="Job",
            metadata=client.V1ObjectMeta(generate_name="mlcube-" +
                                         self.mlcube.name + "-"),
            spec=job_spec,
        )

        logging.info("The MLCube Kubernetes Job manifest %s" %
                     self.mlcube_job_manifest)

    def create_job(self, k8s_job_client: client.BatchV1Api):
        job_creation_response = k8s_job_client.create_namespaced_job(
            body=self.mlcube_job_manifest,
            namespace=self.namespace,
        )

        logging.info("MLCommons Box k8s job created. Status='%s'" %
                     str(job_creation_response.status))

    def run(self):
        """Run a cube"""
        logging.info("Configuring MLCube as a Kubernetes Job...")
        self.create_job_manifest()

        config.load_kube_config()
        batch_v1 = client.BatchV1Api()
        # create job on k8s cluster
        self.create_job(batch_v1)



from click.testing import CliRunner
from mlcube_k8s.main import cli

runner = CliRunner()


def test_mlcube_k8s():
    response = runner.invoke(cli)
    assert response.exit_code == 0
    assert 'Usage: mlcube_k8s [OPTIONS] COMMAND [ARGS]...' in response.output
    assert "  run  Runs a MLCube in a Kubernetes cluster." in response.output



import os
from unittest import TestCase
from mlcube.common import mlcube_metadata, objects
from mlcube.common.objects import platform_config
from mlcube_k8s.k8s_run import KubernetesRun


class TestKubernetesRun(TestCase):
    def setUp(self):
        self.path_to_mlcube = os.path.join(os.path.dirname(__file__), "test_data/test_cube")
        self.path_to_platform = os.path.join(self.path_to_mlcube, "platforms/docker.yaml")
        self.path_to_task = os.path.join(self.path_to_mlcube, "run/kubernetes.yaml")
        self.mlcube = mlcube_metadata.MLCube(path=self.path_to_mlcube)
        self.mlcube.platform = objects.load_object_from_file(
            file_path=self.path_to_platform,
            obj_class=platform_config.PlatformConfig)
        self.mlcube.invoke = mlcube_metadata.MLCubeInvoke(self.path_to_task)
        self.mlcube_k8s_runner = KubernetesRun(mlcube=self.mlcube,
                                              loglevel="INFO")

    def test_create_job_manifest(self):
        self.mlcube_k8s_runner.create_job_manifest()
        assert True



import logging
import click

from mlcube import parse  # Do not remove (it registers schemas on import)
from mlcube.common import mlcube_metadata, objects
from mlcube.common.objects import platform_config
from mlcube_k8s.k8s_run import KubernetesRun


@click.group(name='mlcube_k8s')
def cli():
    """
    MLCube k8s Runner runs cubes (packaged Machine Learning workloads) in a Kubernetes cluster.
    """
    pass


@cli.command(name="run")
@click.option('--mlcube',
              required=True,
              type=click.Path(exists=True),
              help='Path to MLCube directory.')
@click.option('--platform',
              required=True,
              type=click.Path(exists=True),
              help='Path to MLCube Platform definition file.')
@click.option('--task',
              required=True,
              type=click.Path(exists=True),
              help='Path to MLCube Task definition file.')
@click.option('--loglevel',
              required=False,
              type=click.STRING,
              default="DEBUG",
              help='Log level for the CLI app.')
def run(mlcube: str, platform: str, task: str, loglevel: str):
    """
    Runs a MLCube in a Kubernetes cluster.
    """
    # set loglevel for CLI
    logging.basicConfig(format='%(asctime)s - %(message)s',
                        datefmt='%d-%b-%y-%H:%M:%S',
                        level=loglevel)
    logger = logging.getLogger(__name__)

    logger.info("Configuring MLCube platform, task for Kubernetes...")
    mlcube: mlcube_metadata.MLCube = mlcube_metadata.MLCube(path=mlcube)
    mlcube.platform = objects.load_object_from_file(
        file_path=platform, obj_class=platform_config.PlatformConfig)
    mlcube.invoke = mlcube_metadata.MLCubeInvoke(task)
    logger.info("MLCube: %s", mlcube)

    runner = KubernetesRun(mlcube, loglevel)
    runner.run()
